@@ -3,6 +3,7 @@ from .binning import ContinuousOptimalBinning
 from .binning import MDLP
 from .binning import MulticlassOptimalBinning
 from .binning import OptimalBinning
+from .binning.distributed import BinningProcessSketch
 from .binning.distributed import OptimalBinningSketch
 from .binning.piecewise import ContinuousOptimalPWBinning
 from .binning.piecewise import OptimalPWBinning
@@ -11,6 +12,7 @@ from .scorecard import Scorecard
 
 
 __all__ = ['BinningProcess',
+           'BinningProcessSketch',
            'ContinuousOptimalBinning',
            'ContinuousOptimalPWBinning',
            'MDLP',
