@@ -28,6 +28,8 @@ from .binning import OptimalBinning
 from .binning_process_information import print_binning_process_information
 from .continuous_binning import ContinuousOptimalBinning
 from .multiclass_binning import MulticlassOptimalBinning
+from .piecewise.binning import OptimalPWBinning
+from .piecewise.continuous_binning import ContinuousOptimalPWBinning
 
 
 _METRICS = {
@@ -49,6 +51,13 @@ _METRICS = {
 }
 
 
+_OPTB_TYPES = (OptimalBinning, ContinuousOptimalBinning,
+               MulticlassOptimalBinning)
+
+
+_OPTBPW_TYPES = (OptimalPWBinning, ContinuousOptimalPWBinning)
+
+
 def _effective_n_jobs(n_jobs):
     # Joblib check
     if n_jobs is None:
@@ -291,10 +300,121 @@ def _check_parameters(variable_names, max_n_prebins, min_prebin_size,
 
 
 def _check_variable_dtype(x):
-    return "categorical" if x.dtype == np.object else "numerical"
+    return "categorical" if x.dtype == object else "numerical"
 
 
-class BinningProcess(Base, BaseEstimator):
+class BaseBinningProcess:
+    @classmethod
+    def load(cls, path):
+        """Load binning process from pickle file.
+
+        Parameters
+        ----------
+        path : str
+            Pickle file path.
+
+        Example
+        -------
+        >>> from optbinning import BinningProcess
+        >>> binning_process = BinningProcess.load("my_binning_process.pkl")
+        """
+        if not isinstance(path, str):
+            raise TypeError("path must be a string.")
+
+        with open(path, "rb") as f:
+            return pickle.load(f)
+
+    def save(self, path):
+        """Save binning process to pickle file.
+
+        Parameters
+        ----------
+        path : str
+            Pickle file path.
+        """
+        if not isinstance(path, str):
+            raise TypeError("path must be a string.")
+
+        with open(path, "wb") as f:
+            pickle.dump(self, f)
+
+    def _support_selection_criteria(self):
+        self._support = np.full(self._n_variables, True, dtype=bool)
+
+        if self.selection_criteria is None:
+            return
+
+        default_metrics_info = _METRICS[self._target_dtype]
+        criteria_metrics = self.selection_criteria.keys()
+
+        binning_metrics = pd.DataFrame.from_dict(self._variable_stats).T
+
+        for metric in default_metrics_info["metrics"]:
+            if metric in criteria_metrics:
+                metric_info = self.selection_criteria[metric]
+                metric_values = binning_metrics[metric].values
+
+                if "min" in metric_info:
+                    self._support &= metric_values >= metric_info["min"]
+                if "max" in metric_info:
+                    self._support &= metric_values <= metric_info["max"]
+                if all(m in metric_info for m in ("strategy", "top")):
+                    indices_valid = np.where(self._support)[0]
+                    metric_values = metric_values[indices_valid]
+                    n_valid = len(metric_values)
+
+                    # Auxiliary support
+                    support = np.full(self._n_variables, False, dtype=bool)
+
+                    top = metric_info["top"]
+                    if not isinstance(top, numbers.Integral):
+                        top = int(np.ceil(n_valid * top))
+                    n_selected = min(n_valid, top)
+
+                    if metric_info["strategy"] == "highest":
+                        mask = np.argsort(-metric_values)[:n_selected]
+                    elif metric_info["strategy"] == "lowest":
+                        mask = np.argsort(metric_values)[:n_selected]
+
+                    support[indices_valid[mask]] = True
+                    self._support &= support
+
+    def _binning_selection_criteria(self):
+        for i, name in enumerate(self.variable_names):
+            optb = self._binned_variables[name]
+            optb.binning_table.build()
+
+            n_bins = len(optb.splits)
+            if optb.dtype == "numerical":
+                n_bins += 1
+
+            info = {"dtype": optb.dtype,
+                    "status": optb.status,
+                    "n_bins": n_bins}
+
+            if self._target_dtype in ("binary", "multiclass"):
+                optb.binning_table.analysis(print_output=False)
+
+                if self._target_dtype == "binary":
+                    metrics = {
+                        "iv": optb.binning_table.iv,
+                        "gini": optb.binning_table.gini,
+                        "js": optb.binning_table.js,
+                        "quality_score": optb.binning_table.quality_score}
+                else:
+                    metrics = {
+                        "js": optb.binning_table.js,
+                        "quality_score": optb.binning_table.quality_score}
+            elif self._target_dtype == "continuous":
+                metrics = {}
+
+            info = {**info, **metrics}
+            self._variable_stats[name] = info
+
+        self._support_selection_criteria()
+
+
+class BinningProcess(Base, BaseEstimator, BaseBinningProcess):
     """Binning process to compute optimal binning of variables in a dataset,
     given a binary, continuous or multiclass target dtype.
 
@@ -394,6 +514,13 @@ class BinningProcess(Base, BaseEstimator):
     * keys ``min`` and ``max`` support numerical values.
     * key ``strategy`` supports options "highest" and "lowest".
     * key ``top`` supports an integer or decimal (percentage).
+
+
+    .. warning::
+
+        If the binning process instance is going to be saved, do not pass the
+        option ``"solver": "mip"`` via the binning_fit_params parameter.
+
     """
     def __init__(self, variable_names, max_n_prebins=20, min_prebin_size=0.05,
                  min_n_bins=None, max_n_bins=None, min_bin_size=None,
@@ -656,7 +783,8 @@ class BinningProcess(Base, BaseEstimator):
 
         Returns
         -------
-        X_new : numpy array, shape = (n_samples, n_features_new)
+        X_new : numpy array or pandas.DataFrame, shape = (n_samples,
+        n_features_new)
             Transformed array.
         """
         self._check_is_fitted()
@@ -786,6 +914,60 @@ class BinningProcess(Base, BaseEstimator):
             raise ValueError("name {} does not match a binned variable."
                              .format(name))
 
+    def update_binned_variable(self, name, optb):
+        """Update optimal binning object for a given variable.
+
+        Parameters
+        ----------
+        name : string
+            The variable name.
+
+        optb : object
+            The optimal binning object already fitted.
+        """
+        self._check_is_fitted()
+
+        if not isinstance(name, str):
+            raise TypeError("name must be a string.")
+
+        if name not in self.variable_names:
+            raise ValueError("name {} does not match a binned variable."
+                             .format(name))
+
+        optb_types = _OPTB_TYPES + _OPTBPW_TYPES
+
+        if not isinstance(optb, optb_types):
+            raise TypeError("Object {} must be of type ({}); got {}"
+                            .format(name, optb_types, type(optb)))
+
+        # Check current class
+        if self._target_dtype == "binary":
+            optb_binary = (OptimalBinning, OptimalPWBinning)
+            if not isinstance(optb, optb_binary):
+                raise TypeError("target is binary and Object {} must be of "
+                                "type {}.".format(optb, optb_binary))
+        elif self._target_dtype == "continuous":
+            optb_continuous = (ContinuousOptimalBinning,
+                               ContinuousOptimalPWBinning)
+            if not isinstance(optb, optb_continuous):
+                raise TypeError("target is continuous and Object {} must be "
+                                "of type {}.".format(optb, optb_continuous))
+        elif self._target_dtype == "multiclass":
+            if not isinstance(optb, MulticlassOptimalBinning):
+                raise TypeError("target is multiclass and Object {} must be "
+                                "of type {}.".format(
+                                    optb, MulticlassOptimalBinning))
+
+        optb_old = self._binned_variables[name]
+        if optb_old.name and optb_old.name != optb.name:
+            raise ValueError("Update object name must match old object name; "
+                             "{} != {}.".format(optb_old.name, optb.name))
+
+        if optb.name and name != optb.name:
+            raise ValueError("name and object name must coincide.")
+
+        self._binned_variables[name] = optb
+
     def get_support(self, indices=False, names=False):
         """Get a mask, or integer index, or names of the variables selected.
 
@@ -824,115 +1006,6 @@ class BinningProcess(Base, BaseEstimator):
         else:
             return mask
 
-    @classmethod
-    def load(cls, path):
-        """Load binning process from pickle file.
-
-        Parameters
-        ----------
-        path : str
-            Pickle file path.
-
-        Example
-        -------
-        >>> from optbinning import BinningProcess
-        >>> binning_process = BinningProcess.load("my_binning_process.pkl")
-        """
-        if not isinstance(path, str):
-            raise TypeError("path must be a string.")
-
-        with open(path, "rb") as f:
-            return pickle.load(f)
-
-    def save(self, path):
-        """Save binning process to pickle file.
-
-        Parameters
-        ----------
-        path : str
-            Pickle file path.
-        """
-        if not isinstance(path, str):
-            raise TypeError("path must be a string.")
-
-        with open(path, "wb") as f:
-            pickle.dump(self, f)
-
-    def _support_selection_criteria(self):
-        self._support = np.full(self._n_variables, True, dtype=np.bool)
-
-        if self.selection_criteria is None:
-            return
-
-        default_metrics_info = _METRICS[self._target_dtype]
-        criteria_metrics = self.selection_criteria.keys()
-
-        binning_metrics = pd.DataFrame.from_dict(self._variable_stats).T
-
-        for metric in default_metrics_info["metrics"]:
-            if metric in criteria_metrics:
-                metric_info = self.selection_criteria[metric]
-                metric_values = binning_metrics[metric].values
-
-                if "min" in metric_info:
-                    self._support &= metric_values >= metric_info["min"]
-                if "max" in metric_info:
-                    self._support &= metric_values <= metric_info["max"]
-                if all(m in metric_info for m in ("strategy", "top")):
-                    indices_valid = np.where(self._support)[0]
-                    metric_values = metric_values[indices_valid]
-                    n_valid = len(metric_values)
-
-                    # Auxiliary support
-                    support = np.full(self._n_variables, False, dtype=np.bool)
-
-                    top = metric_info["top"]
-                    if not isinstance(top, numbers.Integral):
-                        top = int(np.ceil(n_valid * top))
-                    n_selected = min(n_valid, top)
-
-                    if metric_info["strategy"] == "highest":
-                        mask = np.argsort(-metric_values)[:n_selected]
-                    elif metric_info["strategy"] == "lowest":
-                        mask = np.argsort(metric_values)[:n_selected]
-
-                    support[indices_valid[mask]] = True
-                    self._support &= support
-
-    def _binning_selection_criteria(self):
-        for i, name in enumerate(self.variable_names):
-            optb = self._binned_variables[name]
-            optb.binning_table.build()
-
-            n_bins = len(optb.splits)
-            if optb.dtype == "numerical":
-                n_bins += 1
-
-            info = {"dtype": optb.dtype,
-                    "status": optb.status,
-                    "n_bins": n_bins}
-
-            if self._target_dtype in ("binary", "multiclass"):
-                optb.binning_table.analysis(print_output=False)
-
-                if self._target_dtype == "binary":
-                    metrics = {
-                        "iv": optb.binning_table.iv,
-                        "gini": optb.binning_table.gini,
-                        "js": optb.binning_table.js,
-                        "quality_score": optb.binning_table.quality_score}
-                else:
-                    metrics = {
-                        "js": optb.binning_table.js,
-                        "quality_score": optb.binning_table.quality_score}
-            elif self._target_dtype == "continuous":
-                metrics = {}
-
-            info = {**info, **metrics}
-            self._variable_stats[name] = info
-
-        self._support_selection_criteria()
-
     def _fit(self, X, y, check_input):
         time_init = time.perf_counter()
 
@@ -989,6 +1062,10 @@ class BinningProcess(Base, BaseEstimator):
 
         if n_jobs == 1:
             for i, name in enumerate(self.variable_names):
+                if self.verbose:
+                    self._logger.info("Binning variable ({} / {}): {}."
+                                      .format(i, self._n_variables, name))
+
                 if isinstance(X, np.ndarray):
                     dtype, optb = _fit_variable(
                         X[:, i], y, name, self._target_dtype,
@@ -1147,15 +1224,13 @@ class BinningProcess(Base, BaseEstimator):
         if not isinstance(dict_optb, dict):
             raise TypeError("dict_optb must be a dict.")
 
-        optb_types = (OptimalBinning, ContinuousOptimalBinning,
-                      MulticlassOptimalBinning)
-
         # Check variable names
         if set(dict_optb.keys()) != set(self.variable_names):
             raise ValueError("dict_optb keys and variable names must "
                              "coincide.")
 
         # Check objects class
+        optb_types = _OPTB_TYPES
         types = set()
         for name, optb in dict_optb.items():
             if not isinstance(name, str):
@@ -1220,7 +1295,7 @@ class BinningProcess(Base, BaseEstimator):
     def _transform(self, X, metric, metric_special, metric_missing,
                    show_digits, check_input):
 
-        # check X dtype
+        # Check X dtype
         if not isinstance(X, (pd.DataFrame, np.ndarray)):
             raise TypeError("X must be a pandas.DataFrame or numpy.ndarray.")
 
@@ -1243,15 +1318,22 @@ class BinningProcess(Base, BaseEstimator):
                     raise ValueError("Selected variable {} must be a column "
                                      "in the input dataframe.".format(name))
 
+        # Check metric
+        if metric in ("indices", "bins"):
+            if any(isinstance(optb, _OPTBPW_TYPES)
+                   for optb in self._binned_variables.values()):
+                raise TypeError("metric {} not supported for piecewise "
+                                "optimal binning objects.".format(metric))
+
         indices_selected_variables = self.get_support(indices=True)
         n_selected_variables = len(indices_selected_variables)
 
         if metric == "indices":
             X_transform = np.full(
-                (n_samples, n_selected_variables), -1, dtype=np.int)
+                (n_samples, n_selected_variables), -1, dtype=int)
         elif metric == "bins":
             X_transform = np.full(
-                (n_samples, n_selected_variables), "", dtype=np.object)
+                (n_samples, n_selected_variables), "", dtype=object)
         else:
             X_transform = np.zeros((n_samples, n_selected_variables))
 
@@ -1259,6 +1341,11 @@ class BinningProcess(Base, BaseEstimator):
             name = self.variable_names[idx]
             optb = self._binned_variables[name]
 
+            if isinstance(X, np.ndarray):
+                x = X[:, idx]
+            else:
+                x = X[name]
+
             params = {}
             if self.binning_transform_params is not None:
                 params = self.binning_transform_params.get(name, {})
@@ -1266,24 +1353,24 @@ class BinningProcess(Base, BaseEstimator):
             metric_missing = params.get("metric_missing", metric_missing)
             metric_special = params.get("metric_special", metric_special)
 
-            if isinstance(X, np.ndarray):
-                x = X[:, idx]
-            else:
-                x = X[name]
+            tparams = {
+                "x": x,
+                "metric": metric,
+                "metric_special": metric_special,
+                "metric_missing": metric_missing,
+                "check_input": check_input,
+                "show_digits": show_digits
+                }
 
-            if metric is None:
-                # Use default metric for each target type
-                X_transform[:, i] = optb.transform(
-                    x=x, metric_special=metric_special,
-                    metric_missing=metric_missing, show_digits=show_digits,
-                    check_input=check_input)
-            else:
-                _metric = params.get("metric", metric)
+            if isinstance(optb, _OPTBPW_TYPES):
+                tparams.pop("show_digits")
 
-                X_transform[:, i] = optb.transform(
-                    x=x, metric=_metric, metric_special=metric_special,
-                    metric_missing=metric_missing, show_digits=show_digits,
-                    check_input=check_input)
+            if metric is not None:
+                tparams["metric"] = params.get("metric", metric)
+            else:
+                tparams.pop("metric")
+
+            X_transform[:, i] = optb.transform(**tparams)
 
         if isinstance(X, pd.DataFrame):
             return pd.DataFrame(X_transform, columns=selected_variables)
@@ -1305,6 +1392,13 @@ class BinningProcess(Base, BaseEstimator):
             raise ValueError("chunksize must be a positive integer; got {}."
                              .format(chunksize))
 
+        # Check metric
+        if metric in ("indices", "bins"):
+            if any(isinstance(optb, _OPTBPW_TYPES)
+                   for optb in self._binned_variables.values()):
+                raise TypeError("metric {} not supported for piecewise "
+                                "optimal binning objects.".format(metric))
+
         selected_variables = self.get_support(names=True)
         n_selected_variables = len(selected_variables)
 
@@ -1316,10 +1410,10 @@ class BinningProcess(Base, BaseEstimator):
 
             if metric == "indices":
                 X_transform = np.full(
-                    (n_samples, n_selected_variables), -1, dtype=np.int)
+                    (n_samples, n_selected_variables), -1, dtype=int)
             elif metric == "bins":
                 X_transform = np.full(
-                    (n_samples, n_selected_variables), "", dtype=np.object)
+                    (n_samples, n_selected_variables), "", dtype=object)
             else:
                 X_transform = np.zeros((n_samples, n_selected_variables))
 
@@ -1333,19 +1427,23 @@ class BinningProcess(Base, BaseEstimator):
                 metric_missing = params.get("metric_missing", metric_missing)
                 metric_special = params.get("metric_special", metric_special)
 
-                x = chunk[name]
+                tparams = {
+                    "x": chunk[name],
+                    "metric": metric,
+                    "metric_special": metric_special,
+                    "metric_missing": metric_missing,
+                    "show_digits": show_digits
+                    }
 
-                if metric is None:
-                    # Use default metric for each target type
-                    X_transform[:, i] = optb.transform(
-                        x=x, metric_special=metric_special,
-                        metric_missing=metric_missing, show_digits=show_digits)
+                if isinstance(optb, _OPTBPW_TYPES):
+                    tparams.pop("show_digits")
+
+                if metric is not None:
+                    tparams["metric"] = params.get("metric", metric)
                 else:
-                    _metric = params.get("metric", metric)
+                    tparams.pop("metric")
 
-                    X_transform[:, i] = optb.transform(
-                        x, _metric, metric_special, metric_missing,
-                        show_digits)
+                X_transform[:, i] = optb.transform(**tparams)
 
             df = pd.DataFrame(X_transform, columns=selected_variables)
             df.to_csv(output_path, mode='a', index=False, header=(k == 0))
