@@ -690,25 +690,26 @@ class ContinuousOptimalBinning(OptimalBinning):
 
         time_init = time.perf_counter()
 
-        if not len(n_records):
+        if len(n_records) <= 1:
             self._status = "OPTIMAL"
             self._splits_optimal = splits
-            self._solution = np.zeros(len(splits)).astype(np.bool)
+            self._solution = np.zeros(len(splits)).astype(bool)
 
             if self.verbose:
-                self._logger.warning("Optimizer: no bins after pre-binning.")
+                self._logger.warning("Optimizer: {} bins after pre-binning."
+                                     .format(len(n_records)))
                 self._logger.warning("Optimizer: solver not run.")
 
                 self._logger.info("Optimizer terminated. Time: 0s")
             return
 
         if self.min_bin_size is not None:
-            min_bin_size = np.int(np.ceil(self.min_bin_size * self._n_samples))
+            min_bin_size = int(np.ceil(self.min_bin_size * self._n_samples))
         else:
             min_bin_size = self.min_bin_size
 
         if self.max_bin_size is not None:
-            max_bin_size = np.int(np.ceil(self.max_bin_size * self._n_samples))
+            max_bin_size = int(np.ceil(self.max_bin_size * self._n_samples))
         else:
             max_bin_size = self.max_bin_size
 
