@@ -9,7 +9,11 @@ Model data for optimal binning formulations.
 import numpy as np
 
 from scipy import stats
-from scipy import special
+
+from .metrics import jeffrey
+from .metrics import jensen_shannon
+from .metrics import hellinger
+from .metrics import triangular
 
 
 def test_proportions(e1, ne1, e2, ne2, zscore):
@@ -89,7 +93,7 @@ def find_pvalue_violation_indices_continuous(n, U, S, R, max_pvalue,
     return pvalue_violation_indices
 
 
-def model_data(n_nonevent, n_event, max_pvalue, max_pvalue_policy,
+def model_data(divergence, n_nonevent, n_event, max_pvalue, max_pvalue_policy,
                scale=None, return_nonevent_event=False):
     n = len(n_nonevent)
 
@@ -109,7 +113,15 @@ def model_data(n_nonevent, n_event, max_pvalue, max_pvalue_policy,
 
         p = s_event / t_n_event
         q = s_nonevent / t_n_nonevent
-        iv = special.xlogy(p - q, p / q)
+
+        if divergence == "iv":
+            iv = jeffrey(p, q)
+        elif divergence == "js":
+            iv = jensen_shannon(p, q)
+        elif divergence == "hellinger":
+            iv = hellinger(p, q)
+        elif divergence == "triangular":
+            iv = triangular(p, q)
 
         if scale is not None:
             rate *= scale
@@ -163,7 +175,7 @@ def multiclass_model_data(n_nonevent, n_event, max_pvalue, max_pvalue_policy,
 
             p = s_event / t_n_event
             q = s_nonevent / t_n_nonevent
-            iv = special.xlogy(p - q, p / q)
+            iv = jeffrey(p, q)
 
             if scale is not None:
                 rate *= scale
@@ -192,7 +204,7 @@ def multiclass_model_data(n_nonevent, n_event, max_pvalue, max_pvalue_policy,
     return DD, VV, PV
 
 
-def continuous_model_data(n_records, sums, stds, max_pvalue,
+def continuous_model_data(n_records, sums, ssums, max_pvalue,
                           max_pvalue_policy, scale=None):
 
     n = len(n_records)
@@ -206,10 +218,11 @@ def continuous_model_data(n_records, sums, stds, max_pvalue,
     for i in range(1, n + 1):
         s_n_records = n_records[:i][::-1].cumsum()[::-1]
         s_sums = sums[:i][::-1].cumsum()[::-1]
-        s_stds = stds[:i][::-1].cumsum()[::-1]
+        s_ssums = ssums[:i][::-1].cumsum()[::-1]
+
         mean = s_sums / s_n_records
+        std = np.sqrt(s_ssums / s_n_records - (s_sums / s_n_records) ** 2)
         norm = np.absolute(sums[i-1] - s_sums)
-        mean_std = s_stds / s_n_records
 
         if scale is not None:
             mean_scaled = mean * scale
@@ -227,7 +240,7 @@ def continuous_model_data(n_records, sums, stds, max_pvalue,
         if max_pvalue is not None:
             UP.append(mean)
             R.append(s_n_records)
-            S.append(mean_std)
+            S.append(std)
 
     if max_pvalue is not None:
         pvalue_violation_indices = find_pvalue_violation_indices_continuous(
