@@ -584,7 +584,7 @@ class MulticlassOptimalBinning(OptimalBinning):
             n_event = np.empty(self._n_classes).astype(np.int64)
 
             for i, cl in enumerate(self._classes):
-                n_event[i] = target_info(y_clean, cl)[1]
+                n_event[i] = target_info(y_clean, cl)[0]
 
         self._n_event = multiclass_bin_info(
             self._solution, self._n_classes, n_event, self._n_event_missing,
@@ -627,8 +627,8 @@ class MulticlassOptimalBinning(OptimalBinning):
         special_target_info = []
         missing_target_info = []
         for idx, cl in enumerate(self._classes):
-            special_target_info.append(target_info(y_special, cl)[1])
-            missing_target_info.append(target_info(y_missing, cl)[1])
+            special_target_info.append(target_info(y_special, cl)[0])
+            missing_target_info.append(target_info(y_missing, cl)[0])
 
         self._n_event_special = special_target_info
         self._n_event_missing = missing_target_info
@@ -655,7 +655,7 @@ class MulticlassOptimalBinning(OptimalBinning):
         if not len(n_nonevent):
             self._status = "OPTIMAL"
             self._splits_optimal = splits
-            self._solution = np.zeros(len(splits)).astype(np.bool)
+            self._solution = np.zeros(len(splits)).astype(bool)
 
             if self.verbose:
                 self._logger.warning("Optimizer: no bins after pre-binning.")
@@ -665,12 +665,12 @@ class MulticlassOptimalBinning(OptimalBinning):
             return
 
         if self.min_bin_size is not None:
-            min_bin_size = np.int(np.ceil(self.min_bin_size * self._n_samples))
+            min_bin_size = int(np.ceil(self.min_bin_size * self._n_samples))
         else:
             min_bin_size = self.min_bin_size
 
         if self.max_bin_size is not None:
-            max_bin_size = np.int(np.ceil(self.max_bin_size * self._n_samples))
+            max_bin_size = int(np.ceil(self.max_bin_size * self._n_samples))
         else:
             max_bin_size = self.max_bin_size
 
@@ -771,7 +771,7 @@ class MulticlassOptimalBinning(OptimalBinning):
         n_bins = n_splits + 1
         n_nonevent = np.empty((n_bins, self._n_classes)).astype(np.int64)
         n_event = np.empty((n_bins, self._n_classes)).astype(np.int64)
-        mask_remove = np.zeros(n_bins).astype(np.bool)
+        mask_remove = np.zeros(n_bins).astype(bool)
 
         for idx, cl in enumerate(self._classes):
             y1 = (y == cl)
